import {
    Bar,
    BarChart,
    CartesianGrid,
    Label,
    Rectangle,
    ResponsiveContainer,
    Tooltip,
    XAxis,
    YAxis,
} from "recharts";
import styled from "styled-components";
import { breakpointNettbrett } from "../../utils/constants";
import { Heading } from "../../components/fellesKomponenter/typografi/Heading";
import { Hendelser } from "../forsinkelseSide/useHentForsinkelsesdata";
import {
    hentForsinkelseIÅrPerDato,
    sumForsinkelseIÅrPerMåned,
    hentTicksYaxisForsinkelse,
    sumForsinkelseITimerPerDato,
} from "./utils";
import React, { useMemo, useState } from "react";
import { BarplotTooltip } from "./BarplotTooltip";
import { BarplotValg } from "./BarplotValg";

interface BarplotForsinkelseProps {
    data: Hendelser[];
}

export enum Tidsenhet {
    Dato = "Dato",
    Måned = "Måned",
}
export const BarplotForsinkelse = ({ data }: BarplotForsinkelseProps) => {
    const [tidsenhet, setTidsenhet] = useState<Tidsenhet>(Tidsenhet.Dato);

    const forsinkelsePerDato = data
        .map((hendelse) => hendelse.forsinkelseTotal.forsinkelsePerDato)
        .flat();
    const aggregertDato = sumForsinkelseITimerPerDato(forsinkelsePerDato);
    const forsinkelseIÅrPerDato = hentForsinkelseIÅrPerDato(aggregertDato);
    const forsinkelseIÅrPerMåned = sumForsinkelseIÅrPerMåned(forsinkelseIÅrPerDato);

    const customTooltip = <BarplotTooltip tidsenhet={tidsenhet} />;
    const visning = useMemo(() => {
        return tidsenhet === Tidsenhet.Dato ? forsinkelseIÅrPerDato : forsinkelseIÅrPerMåned;
    }, [tidsenhet, data]);
    const tickArrayYaxis = useMemo(() => {
        return hentTicksYaxisForsinkelse(visning.map((d) => d.forsinkelse));
    }, [tidsenhet, data]);

    return (
        <div>
            <GrafContainer>
                <CustomHeading styleLevel={"4"} as={"h4"}>
                    Forsinkelse per dato
                </CustomHeading>
                <ResponsiveContainer height={360} minWidth={550}>
                    <BarChart
                        margin={{ top: 50, right: 40, left: 20, bottom: 30 }}
                        data={visning}
                        barSize={30}
                    >
                        <CartesianGrid strokeDasharray="3 3" />
                        <XAxis dataKey="dato" user-select={"none"}>
                            <Label value={"Dato"} offset={-15} position="insideBottom" />
                        </XAxis>
                        <YAxis domain={[0, Math.max(...tickArrayYaxis)]} ticks={tickArrayYaxis}>
                            <Label
                                value={"Forsinkelse (år)"}
                                offset={15}
                                position="insideBottomLeft"
                                angle={-90}
                            />
                        </YAxis>
                        <Tooltip content={customTooltip} />
                        <Bar
                            dataKey="forsinkelse"
                            fill={barColor}
                            activeBar={<Rectangle fill={activeBarColor} />}
                        />
                    </BarChart>
                </ResponsiveContainer>
                <BarplotValg tidsenhet={tidsenhet} setTidsenhet={setTidsenhet} />
            </GrafContainer>
        </div>
    );
};

const activeBarColor = "#0786b4";
const barColor = "#076d91";

const GrafContainer = styled.div`
    @media (max-width: ${breakpointNettbrett}) {
        overflow-x: scroll;
        overflow-y: hidden;
    }
`;

const CustomHeading = styled(Heading)`
    display: flex;
    justify-content: center;
    margin-bottom: 10px;
`;

export default BarplotForsinkelse;
